@@ -15,24 +15,6 @@
 //
 //
 
-#include <limits.h>
-#include <string.h>
-
-#include <algorithm>
-#include <atomic>
-#include <cstdlib>
-#include <memory>
-#include <new>
-#include <sstream>
-#include <string>
-#include <type_traits>
-#include <utility>
-#include <vector>
-
-#include "absl/log/check.h"
-#include "absl/log/log.h"
-#include "absl/status/status.h"
-
 #include <grpc/byte_buffer.h>
 #include <grpc/grpc.h>
 #include <grpc/impl/channel_arg_names.h>
@@ -67,14 +49,30 @@
 #include <grpcpp/support/server_interceptor.h>
 #include <grpcpp/support/slice.h>
 #include <grpcpp/support/status.h>
+#include <limits.h>
+#include <string.h>
 
+#include <algorithm>
+#include <atomic>
+#include <cstdlib>
+#include <memory>
+#include <new>
+#include <sstream>
+#include <string>
+#include <type_traits>
+#include <utility>
+#include <vector>
+
+#include "absl/log/check.h"
+#include "absl/log/log.h"
+#include "absl/status/status.h"
 #include "src/core/ext/transport/inproc/inproc_transport.h"
-#include "src/core/lib/gprpp/manual_constructor.h"
 #include "src/core/lib/iomgr/exec_ctx.h"
 #include "src/core/lib/iomgr/iomgr.h"
 #include "src/core/lib/resource_quota/api.h"
 #include "src/core/lib/surface/completion_queue.h"
 #include "src/core/server/server.h"
+#include "src/core/util/manual_constructor.h"
 #include "src/cpp/client/create_channel_internal.h"
 #include "src/cpp/server/external_connection_acceptor_impl.h"
 #include "src/cpp/server/health/default_health_check_service.h"
@@ -107,11 +105,18 @@ class DefaultGlobalCallbacks final : public Server::GlobalCallbacks {
 };
 
 std::shared_ptr<Server::GlobalCallbacks> g_callbacks = nullptr;
+Server::GlobalCallbacks* g_raw_callbacks = nullptr;
 gpr_once g_once_init_callbacks = GPR_ONCE_INIT;
 
 void InitGlobalCallbacks() {
-  if (!g_callbacks) {
-    g_callbacks = std::make_shared<DefaultGlobalCallbacks>();
+  if (grpc_core::IsServerGlobalCallbacksOwnershipEnabled()) {
+    if (!g_raw_callbacks) {
+      g_raw_callbacks = new DefaultGlobalCallbacks;
+    }
+  } else {
+    if (!g_callbacks) {
+      g_callbacks = std::make_shared<DefaultGlobalCallbacks>();
+    }
   }
 }
 
@@ -429,7 +434,9 @@ class Server::SyncRequest final : public grpc::internal::CompletionQueueTag {
     ctx_->ctx.cq_ = &cq_;
     request_metadata_.count = 0;
 
-    global_callbacks_ = global_callbacks;
+    if (!grpc_core::IsServerGlobalCallbacksOwnershipEnabled()) {
+      global_callbacks_ = global_callbacks;
+    }
     resources_ = resources;
 
     interceptor_methods_.SetCall(&*wrapped_call_);
@@ -465,13 +472,21 @@ class Server::SyncRequest final : public grpc::internal::CompletionQueueTag {
 
   void ContinueRunAfterInterception() {
     ctx_->ctx.BeginCompletionOp(&*wrapped_call_, nullptr, nullptr);
-    global_callbacks_->PreSynchronousRequest(&ctx_->ctx);
+    if (grpc_core::IsServerGlobalCallbacksOwnershipEnabled()) {
+      g_raw_callbacks->PreSynchronousRequest(&ctx_->ctx);
+    } else {
+      global_callbacks_->PreSynchronousRequest(&ctx_->ctx);
+    }
     auto* handler = resources_ ? method_->handler()
                                : server_->resource_exhausted_handler_.get();
     handler->RunHandler(grpc::internal::MethodHandler::HandlerParameter(
         &*wrapped_call_, &ctx_->ctx, deserialized_request_, request_status_,
         nullptr, nullptr));
-    global_callbacks_->PostSynchronousRequest(&ctx_->ctx);
+    if (grpc_core::IsServerGlobalCallbacksOwnershipEnabled()) {
+      g_raw_callbacks->PostSynchronousRequest(&ctx_->ctx);
+    } else {
+      global_callbacks_->PostSynchronousRequest(&ctx_->ctx);
+    }
 
     cq_.Shutdown();
 
@@ -532,7 +547,7 @@ class Server::SyncRequest final : public grpc::internal::CompletionQueueTag {
   grpc::internal::InterceptorBatchMethodsImpl interceptor_methods_;
 
   // ServerContextWrapper allows ManualConstructor while using a private
-  // contructor of ServerContext via this friend class.
+  // constructor of ServerContext via this friend class.
   struct ServerContextWrapper {
     ServerContext ctx;
 
@@ -911,8 +926,12 @@ Server::Server(
       health_check_service_disabled_(false),
       server_metric_recorder_(server_metric_recorder) {
   gpr_once_init(&grpc::g_once_init_callbacks, grpc::InitGlobalCallbacks);
-  global_callbacks_ = grpc::g_callbacks;
-  global_callbacks_->UpdateArguments(args);
+  if (grpc_core::IsServerGlobalCallbacksOwnershipEnabled()) {
+    g_raw_callbacks->UpdateArguments(args);
+  } else {
+    global_callbacks_ = grpc::g_callbacks;
+    global_callbacks_->UpdateArguments(args);
+  }
 
   if (sync_server_cqs_ != nullptr) {
     bool default_rq_created = false;
@@ -997,9 +1016,15 @@ Server::~Server() {
 }
 
 void Server::SetGlobalCallbacks(GlobalCallbacks* callbacks) {
-  CHECK(!grpc::g_callbacks);
-  CHECK(callbacks);
-  grpc::g_callbacks.reset(callbacks);
+  if (grpc_core::IsServerGlobalCallbacksOwnershipEnabled()) {
+    CHECK(!g_raw_callbacks);
+    CHECK(callbacks);
+    g_raw_callbacks = callbacks;
+  } else {
+    CHECK(!g_callbacks);
+    CHECK(callbacks);
+    g_callbacks.reset(callbacks);
+  }
 }
 
 grpc_server* Server::c_server() { return server_; }
@@ -1126,7 +1151,11 @@ int Server::AddListeningPort(const std::string& addr,
                              grpc::ServerCredentials* creds) {
   CHECK(!started_);
   int port = creds->AddPortToServer(addr, server_);
-  global_callbacks_->AddPort(this, addr, creds, port);
+  if (grpc_core::IsServerGlobalCallbacksOwnershipEnabled()) {
+    g_raw_callbacks->AddPort(this, addr, creds, port);
+  } else {
+    global_callbacks_->AddPort(this, addr, creds, port);
+  }
   return port;
 }
 
@@ -1159,7 +1188,11 @@ void Server::UnrefAndWaitLocked() {
 
 void Server::Start(grpc::ServerCompletionQueue** cqs, size_t num_cqs) {
   CHECK(!started_);
-  global_callbacks_->PreServerStart(this);
+  if (grpc_core::IsServerGlobalCallbacksOwnershipEnabled()) {
+    g_raw_callbacks->PreServerStart(this);
+  } else {
+    g_callbacks->PreServerStart(this);
+  }
   started_ = true;
 
   // Only create default health check service when user did not provide an
