@@ -16,11 +16,12 @@
 import logging
 import unittest
 
+from tests.observability import _from_csm_observability_import_star
 from tests.observability import _from_observability_import_star
 
 
 class AllTest(unittest.TestCase):
-    def testAll(self):
+    def testBaseOtel(self):
         expected_observability_code_elements = ("OpenTelemetryPlugin",)
 
         self.assertCountEqual(
@@ -28,6 +29,14 @@ class AllTest(unittest.TestCase):
             _from_observability_import_star.GRPC_OBSERVABILITY_ELEMENTS,
         )
 
+    def testCSMOtel(self):
+        expected_csm_observability_code_elements = ("CsmOpenTelemetryPlugin",)
+
+        self.assertCountEqual(
+            expected_csm_observability_code_elements,
+            _from_csm_observability_import_star.GRPC_CSM_OBSERVABILITY_ELEMENTS,
+        )
+
 
 if __name__ == "__main__":
     logging.basicConfig()
