@@ -26,6 +26,7 @@
 #include "absl/strings/string_view.h"
 #include "absl/time/time.h"
 #include "gtest/gtest.h"
+#include "src/core/call/metadata_batch.h"
 #include "src/core/config/core_configuration.h"
 #include "src/core/lib/channel/channel_args.h"
 #include "src/core/lib/channel/channel_fwd.h"
@@ -39,7 +40,6 @@
 #include "src/core/lib/slice/slice.h"
 #include "src/core/lib/slice/slice_buffer.h"
 #include "src/core/lib/surface/channel_stack_type.h"
-#include "src/core/lib/transport/metadata_batch.h"
 #include "src/core/lib/transport/transport.h"
 #include "src/core/telemetry/call_tracer.h"
 #include "src/core/telemetry/metrics.h"
@@ -53,18 +53,96 @@
 namespace grpc_core {
 namespace {
 
-Mutex* g_mu;
-CoreEnd2endTest::TestNotification* g_client_call_ended_notify;
-CoreEnd2endTest::TestNotification* g_server_call_ended_notify;
+class TestState {
+ public:
+  explicit TestState(CoreEnd2endTest* test)
+      : client_call_ended_(test), server_call_ended_(test) {}
+
+  void NotifyClient() { client_call_ended_.Notify(); }
+
+  void NotifyServer() { server_call_ended_.Notify(); }
+
+  void WaitForClient() {
+    EXPECT_TRUE(
+        client_call_ended_.WaitForNotificationWithTimeout(absl::Seconds(5)));
+  }
+
+  void WaitForServer() {
+    EXPECT_TRUE(
+        server_call_ended_.WaitForNotificationWithTimeout(absl::Seconds(5)));
+  }
+
+  void ResetClientByteSizes(
+      CallTracerInterface::TransportByteSize incoming = {},
+      CallTracerInterface::TransportByteSize outgoing = {}) {
+    MutexLock lock(&mu_);
+    client_incoming_bytes_ = incoming;
+    client_outgoing_bytes_ = outgoing;
+  }
+
+  void IncrementClientIncomingBytes(
+      CallTracerInterface::TransportByteSize bytes) {
+    MutexLock lock(&mu_);
+    client_incoming_bytes_ += bytes;
+  }
+
+  void IncrementClientOutgoingBytes(
+      CallTracerInterface::TransportByteSize bytes) {
+    MutexLock lock(&mu_);
+    client_outgoing_bytes_ += bytes;
+  }
+
+  void ResetServerByteSizes(
+      CallTracerInterface::TransportByteSize incoming = {},
+      CallTracerInterface::TransportByteSize outgoing = {}) {
+    MutexLock lock(&mu_);
+    server_incoming_bytes_ = incoming;
+    server_outgoing_bytes_ = outgoing;
+  }
+
+  void IncrementServerIncomingBytes(
+      CallTracerInterface::TransportByteSize bytes) {
+    MutexLock lock(&mu_);
+    server_incoming_bytes_ += bytes;
+  }
+
+  void IncrementServerOutgoingBytes(
+      CallTracerInterface::TransportByteSize bytes) {
+    MutexLock lock(&mu_);
+    server_outgoing_bytes_ += bytes;
+  }
+
+  std::tuple<CallTracerInterface::TransportByteSize,
+             CallTracerInterface::TransportByteSize,
+             CallTracerInterface::TransportByteSize,
+             CallTracerInterface::TransportByteSize>
+  ByteSizes() {
+    MutexLock lock(&mu_);
+    return std::tuple(client_incoming_bytes_, client_outgoing_bytes_,
+                      server_incoming_bytes_, server_outgoing_bytes_);
+  }
+
+ private:
+  Mutex mu_;
+  CoreEnd2endTest::TestNotification client_call_ended_;
+  CoreEnd2endTest::TestNotification server_call_ended_;
+  CallTracerInterface::TransportByteSize client_incoming_bytes_
+      ABSL_GUARDED_BY(mu_);
+  CallTracerInterface::TransportByteSize client_outgoing_bytes_
+      ABSL_GUARDED_BY(mu_);
+  CallTracerInterface::TransportByteSize server_incoming_bytes_
+      ABSL_GUARDED_BY(mu_);
+  CallTracerInterface::TransportByteSize server_outgoing_bytes_
+      ABSL_GUARDED_BY(mu_);
+};
 
 class FakeCallTracer : public ClientCallTracer {
  public:
   class FakeCallAttemptTracer : public CallAttemptTracer {
    public:
-    FakeCallAttemptTracer() {
-      MutexLock lock(g_mu);
-      incoming_bytes_ = TransportByteSize();
-      outgoing_bytes_ = TransportByteSize();
+    explicit FakeCallAttemptTracer(std::shared_ptr<TestState> test_state)
+        : test_state_(std::move(test_state)) {
+      test_state_->ResetClientByteSizes();
     }
     std::string TraceId() override { return ""; }
     std::string SpanId() override { return ""; }
@@ -86,38 +164,35 @@ class FakeCallTracer : public ClientCallTracer {
         absl::Status /*status*/,
         grpc_metadata_batch* /*recv_trailing_metadata*/,
         const grpc_transport_stream_stats* transport_stream_stats) override {
-      if (IsCallTracerInTransportEnabled()) return;
-      TransportByteSize incoming_bytes = {
-          transport_stream_stats->incoming.framing_bytes,
-          transport_stream_stats->incoming.data_bytes,
-          transport_stream_stats->incoming.header_bytes};
-      TransportByteSize outgoing_bytes = {
-          transport_stream_stats->outgoing.framing_bytes,
-          transport_stream_stats->outgoing.data_bytes,
-          transport_stream_stats->outgoing.header_bytes};
-      MutexLock lock(g_mu);
-      incoming_bytes_ = incoming_bytes;
-      outgoing_bytes_ = outgoing_bytes;
+      if (IsCallTracerInTransportEnabled() ||
+          transport_stream_stats == nullptr /* cancelled call */) {
+        return;
+      }
+      test_state_->ResetClientByteSizes(
+          {transport_stream_stats->incoming.framing_bytes,
+           transport_stream_stats->incoming.data_bytes,
+           transport_stream_stats->incoming.header_bytes},
+          {transport_stream_stats->outgoing.framing_bytes,
+           transport_stream_stats->outgoing.data_bytes,
+           transport_stream_stats->outgoing.header_bytes});
     }
 
     void RecordIncomingBytes(
         const TransportByteSize& transport_byte_size) override {
-      MutexLock lock(g_mu);
-      incoming_bytes_ += transport_byte_size;
+      test_state_->IncrementClientIncomingBytes(transport_byte_size);
     }
 
     void RecordOutgoingBytes(
         const TransportByteSize& transport_byte_size) override {
-      MutexLock lock(g_mu);
-      outgoing_bytes_ += transport_byte_size;
+      test_state_->IncrementClientOutgoingBytes(transport_byte_size);
     }
 
     void RecordCancel(grpc_error_handle /*cancel_error*/) override {}
-    std::shared_ptr<TcpTracerInterface> StartNewTcpTrace() override {
+    std::shared_ptr<TcpCallTracer> StartNewTcpTrace() override {
       return nullptr;
     }
     void RecordEnd(const gpr_timespec& /*latency*/) override {
-      g_client_call_ended_notify->Notify();
+      test_state_->NotifyClient();
       delete this;
     }
     void RecordAnnotation(absl::string_view /*annotation*/) override {}
@@ -126,22 +201,12 @@ class FakeCallTracer : public ClientCallTracer {
     void SetOptionalLabel(OptionalLabelKey /*key*/,
                           RefCountedStringValue /*value*/) override {}
 
-    static TransportByteSize incoming_bytes() {
-      MutexLock lock(g_mu);
-      return incoming_bytes_;
-    }
-
-    static TransportByteSize outgoing_bytes() {
-      MutexLock lock(g_mu);
-      return outgoing_bytes_;
-    }
-
    private:
-    static TransportByteSize incoming_bytes_ ABSL_GUARDED_BY(g_mu);
-    static TransportByteSize outgoing_bytes_ ABSL_GUARDED_BY(g_mu);
+    std::shared_ptr<TestState> test_state_;
   };
 
-  explicit FakeCallTracer() {}
+  explicit FakeCallTracer(std::shared_ptr<TestState> test_state)
+      : test_state_(std::move(test_state)) {}
   ~FakeCallTracer() override {}
   std::string TraceId() override { return ""; }
   std::string SpanId() override { return ""; }
@@ -149,24 +214,21 @@ class FakeCallTracer : public ClientCallTracer {
 
   FakeCallAttemptTracer* StartNewAttempt(
       bool /*is_transparent_retry*/) override {
-    return new FakeCallAttemptTracer;
+    return new FakeCallAttemptTracer(test_state_);
   }
 
   void RecordAnnotation(absl::string_view /*annotation*/) override {}
   void RecordAnnotation(const Annotation& /*annotation*/) override {}
-};
 
-CallTracerInterface::TransportByteSize
-    FakeCallTracer::FakeCallAttemptTracer::incoming_bytes_;
-CallTracerInterface::TransportByteSize
-    FakeCallTracer::FakeCallAttemptTracer::outgoing_bytes_;
+ private:
+  std::shared_ptr<TestState> test_state_;
+};
 
 class FakeServerCallTracer : public ServerCallTracer {
  public:
-  FakeServerCallTracer() {
-    MutexLock lock(g_mu);
-    incoming_bytes_ = TransportByteSize();
-    outgoing_bytes_ = TransportByteSize();
+  explicit FakeServerCallTracer(std::shared_ptr<TestState> test_state)
+      : test_state_(test_state) {
+    test_state_->ResetServerByteSizes();
   }
   ~FakeServerCallTracer() override {}
   void RecordSendInitialMetadata(
@@ -182,39 +244,31 @@ class FakeServerCallTracer : public ServerCallTracer {
   void RecordReceivedDecompressedMessage(
       const Message& /*recv_decompressed_message*/) override {}
   void RecordCancel(grpc_error_handle /*cancel_error*/) override {}
-  std::shared_ptr<TcpTracerInterface> StartNewTcpTrace() override {
-    return nullptr;
-  }
+  std::shared_ptr<TcpCallTracer> StartNewTcpTrace() override { return nullptr; }
   void RecordReceivedTrailingMetadata(
       grpc_metadata_batch* /*recv_trailing_metadata*/) override {}
 
   void RecordEnd(const grpc_call_final_info* final_info) override {
     if (!IsCallTracerInTransportEnabled()) {
-      TransportByteSize incoming_bytes = {
-          final_info->stats.transport_stream_stats.incoming.framing_bytes,
-          final_info->stats.transport_stream_stats.incoming.data_bytes,
-          final_info->stats.transport_stream_stats.incoming.header_bytes};
-      TransportByteSize outgoing_bytes = {
-          final_info->stats.transport_stream_stats.outgoing.framing_bytes,
-          final_info->stats.transport_stream_stats.outgoing.data_bytes,
-          final_info->stats.transport_stream_stats.outgoing.header_bytes};
-      MutexLock lock(g_mu);
-      incoming_bytes_ = incoming_bytes;
-      outgoing_bytes_ = outgoing_bytes;
+      test_state_->ResetServerByteSizes(
+          {final_info->stats.transport_stream_stats.incoming.framing_bytes,
+           final_info->stats.transport_stream_stats.incoming.data_bytes,
+           final_info->stats.transport_stream_stats.incoming.header_bytes},
+          {final_info->stats.transport_stream_stats.outgoing.framing_bytes,
+           final_info->stats.transport_stream_stats.outgoing.data_bytes,
+           final_info->stats.transport_stream_stats.outgoing.header_bytes});
     }
-    g_server_call_ended_notify->Notify();
+    test_state_->NotifyServer();
   }
 
   void RecordIncomingBytes(
       const TransportByteSize& transport_byte_size) override {
-    MutexLock lock(g_mu);
-    incoming_bytes_ += transport_byte_size;
+    test_state_->IncrementServerIncomingBytes(transport_byte_size);
   }
 
   void RecordOutgoingBytes(
       const TransportByteSize& transport_byte_size) override {
-    MutexLock lock(g_mu);
-    outgoing_bytes_ += transport_byte_size;
+    test_state_->IncrementServerOutgoingBytes(transport_byte_size);
   }
 
   void RecordAnnotation(absl::string_view /*annotation*/) override {}
@@ -223,47 +277,37 @@ class FakeServerCallTracer : public ServerCallTracer {
   std::string SpanId() override { return ""; }
   bool IsSampled() override { return false; }
 
-  static TransportByteSize incoming_bytes() {
-    MutexLock lock(g_mu);
-    return incoming_bytes_;
-  }
-
-  static TransportByteSize outgoing_bytes() {
-    MutexLock lock(g_mu);
-    return outgoing_bytes_;
-  }
-
  private:
-  static TransportByteSize incoming_bytes_ ABSL_GUARDED_BY(g_mu);
-  static TransportByteSize outgoing_bytes_ ABSL_GUARDED_BY(g_mu);
+  std::shared_ptr<TestState> test_state_;
 };
 
-CallTracerInterface::TransportByteSize FakeServerCallTracer::incoming_bytes_;
-CallTracerInterface::TransportByteSize FakeServerCallTracer::outgoing_bytes_;
-
 // TODO(yijiem): figure out how to reuse FakeStatsPlugin instead of
 // inheriting and overriding it here.
 class NewFakeStatsPlugin : public FakeStatsPlugin {
  public:
+  explicit NewFakeStatsPlugin(std::shared_ptr<TestState> test_state)
+      : test_state_(std::move(test_state)) {}
+
   ClientCallTracer* GetClientCallTracer(
       const Slice& /*path*/, bool /*registered_method*/,
       std::shared_ptr<StatsPlugin::ScopeConfig> /*scope_config*/) override {
-    return GetContext<Arena>()->ManagedNew<FakeCallTracer>();
+    return GetContext<Arena>()->ManagedNew<FakeCallTracer>(test_state_);
   }
   ServerCallTracer* GetServerCallTracer(
       std::shared_ptr<StatsPlugin::ScopeConfig> /*scope_config*/) override {
-    return GetContext<Arena>()->ManagedNew<FakeServerCallTracer>();
+    return GetContext<Arena>()->ManagedNew<FakeServerCallTracer>(test_state_);
   }
+
+ private:
+  std::shared_ptr<TestState> test_state_;
 };
 
 // This test verifies the HTTP2 stats on a stream
 CORE_END2END_TEST(Http2FullstackSingleHopTests, StreamStats) {
-  g_mu = new Mutex();
-  g_client_call_ended_notify = new CoreEnd2endTest::TestNotification(this);
-  g_server_call_ended_notify = new CoreEnd2endTest::TestNotification(this);
+  auto test_state = std::make_shared<TestState>(this);
   GlobalStatsPluginRegistryTestPeer::ResetGlobalStatsPluginRegistry();
   GlobalStatsPluginRegistry::RegisterStatsPlugin(
-      std::make_shared<NewFakeStatsPlugin>());
+      std::make_shared<NewFakeStatsPlugin>(test_state));
   auto send_from_client = RandomSlice(10);
   auto send_from_server = RandomSlice(20);
   IncomingStatusOnClient server_status;
@@ -301,15 +345,11 @@ CORE_END2END_TEST(Http2FullstackSingleHopTests, StreamStats) {
   EXPECT_EQ(client_message.payload(), send_from_client);
   EXPECT_EQ(server_message.payload(), send_from_server);
   // Make sure that the calls have ended for the stats to have been collected
-  g_client_call_ended_notify->WaitForNotificationWithTimeout(absl::Seconds(5));
-  g_server_call_ended_notify->WaitForNotificationWithTimeout(absl::Seconds(5));
-
-  auto client_outgoing_transport_stats =
-      FakeCallTracer::FakeCallAttemptTracer::outgoing_bytes();
-  auto client_incoming_transport_stats =
-      FakeCallTracer::FakeCallAttemptTracer::incoming_bytes();
-  auto server_outgoing_transport_stats = FakeServerCallTracer::outgoing_bytes();
-  auto server_incoming_transport_stats = FakeServerCallTracer::incoming_bytes();
+  test_state->WaitForClient();
+  test_state->WaitForServer();
+  auto [client_incoming_transport_stats, client_outgoing_transport_stats,
+        server_incoming_transport_stats, server_outgoing_transport_stats] =
+      test_state->ByteSizes();
   EXPECT_EQ(client_outgoing_transport_stats.data_bytes,
             send_from_client.size());
   EXPECT_EQ(client_incoming_transport_stats.data_bytes,
@@ -330,13 +370,6 @@ CORE_END2END_TEST(Http2FullstackSingleHopTests, StreamStats) {
   EXPECT_LE(server_outgoing_transport_stats.framing_bytes, 58);
   EXPECT_GE(server_incoming_transport_stats.framing_bytes, 32);
   EXPECT_LE(server_incoming_transport_stats.framing_bytes, 58);
-
-  delete g_client_call_ended_notify;
-  g_client_call_ended_notify = nullptr;
-  delete g_server_call_ended_notify;
-  g_server_call_ended_notify = nullptr;
-  delete g_mu;
-  g_mu = nullptr;
 }
 
 }  // namespace
