@@ -14,10 +14,12 @@
 // limitations under the License.
 //
 
+#include <google/protobuf/text_format.h>
 #include <grpc/grpc.h>
 
 #include <map>
 #include <memory>
+#include <optional>
 #include <set>
 #include <string>
 #include <utility>
@@ -28,11 +30,13 @@
 #include "absl/strings/str_cat.h"
 #include "absl/strings/string_view.h"
 #include "absl/time/time.h"
-#include "absl/types/optional.h"
 #include "envoy/service/discovery/v3/discovery.pb.h"
+#include "fuzztest/fuzztest.h"
+#include "gtest/gtest.h"
 #include "src/core/lib/iomgr/timer_manager.h"
 #include "src/core/util/orphanable.h"
 #include "src/core/util/ref_counted_ptr.h"
+#include "src/core/util/wait_for_single_owner.h"
 #include "src/core/xds/grpc/xds_bootstrap_grpc.h"
 #include "src/core/xds/grpc/xds_cluster.h"
 #include "src/core/xds/grpc/xds_cluster_parser.h"
@@ -44,7 +48,6 @@
 #include "src/core/xds/grpc/xds_route_config_parser.h"
 #include "src/core/xds/xds_client/xds_bootstrap.h"
 #include "src/core/xds/xds_client/xds_client.h"
-#include "src/libfuzzer/libfuzzer_macro.h"
 #include "test/core/event_engine/event_engine_test_utils.h"
 #include "test/core/event_engine/fuzzing_event_engine/fuzzing_event_engine.h"
 #include "test/core/xds/xds_client_fuzzer.pb.h"
@@ -84,8 +87,7 @@ class Fuzzer {
     event_engine_->FuzzingDone();
     event_engine_->TickUntilIdle();
     event_engine_->UnsetGlobalHooks();
-    grpc_event_engine::experimental::WaitForSingleOwner(
-        std::move(event_engine_));
+    WaitForSingleOwner(std::move(event_engine_));
     grpc_shutdown_blocking();
   }
 
@@ -189,27 +191,23 @@ class Fuzzer {
         : resource_name_(std::move(resource_name)) {}
 
     void OnResourceChanged(
-        std::shared_ptr<const typename ResourceType::ResourceType> resource,
+        absl::StatusOr<
+            std::shared_ptr<const typename ResourceType::ResourceType>>
+            resource,
         RefCountedPtr<XdsClient::ReadDelayHandle> /* read_delay_handle */)
         override {
       LOG(INFO) << "==> OnResourceChanged(" << ResourceType::Get()->type_url()
-                << " " << resource_name_ << "): " << resource->ToString();
+                << " " << resource_name_ << "): "
+                << (resource.ok() ? (*resource)->ToString()
+                                  : resource.status().ToString());
     }
 
-    void OnError(
+    void OnAmbientError(
         absl::Status status,
         RefCountedPtr<XdsClient::ReadDelayHandle> /* read_delay_handle */)
         override {
-      LOG(INFO) << "==> OnError(" << ResourceType::Get()->type_url() << " "
-                << resource_name_ << "): " << status;
-    }
-
-    void OnResourceDoesNotExist(
-        RefCountedPtr<XdsClient::ReadDelayHandle> /* read_delay_handle */)
-        override {
-      LOG(INFO) << "==> OnResourceDoesNotExist("
-                << ResourceType::Get()->type_url() << " " << resource_name_
-                << ")";
+      LOG(INFO) << "==> OnAmbientError(" << ResourceType::Get()->type_url()
+                << " " << resource_name_ << "): " << status;
     }
 
    private:
@@ -268,11 +266,12 @@ class Fuzzer {
 
   void TriggerConnectionFailure(const std::string& authority,
                                 absl::Status status) {
+    if (status.ok()) return;
     LOG(INFO) << "### TriggerConnectionFailure(" << authority
               << "): " << status;
     const auto* xds_server = GetServer(authority);
     if (xds_server == nullptr) return;
-    transport_factory_->TriggerConnectionFailure(*xds_server,
+    transport_factory_->TriggerConnectionFailure(*xds_server->target(),
                                                  std::move(status));
   }
 
@@ -294,7 +293,7 @@ class Fuzzer {
     if (xds_server == nullptr) return nullptr;
     const char* method = StreamIdMethod(stream_id);
     if (method == nullptr) return nullptr;
-    return transport_factory_->WaitForStream(*xds_server, method);
+    return transport_factory_->WaitForStream(*xds_server->target(), method);
   }
 
   static std::string StreamIdString(
@@ -349,14 +348,314 @@ class Fuzzer {
   std::map<std::string, std::set<EndpointWatcher*>> endpoint_watchers_;
 };
 
-}  // namespace grpc_core
+static const char* kBasicListener = R"pb(
+  bootstrap: "{\"xds_servers\": [{\"server_uri\":\"xds.example.com:443\", \"channel_creds\":[{\"type\": \"fake\"}]}]}"
+  actions {
+    start_watch {
+      resource_type { listener {} }
+      resource_name: "server.example.com"
+    }
+  }
+  actions {
+    read_message_from_client {
+      stream_id { ads {} }
+      ok: true
+    }
+  }
+  actions {
+    send_message_to_client {
+      stream_id { ads {} }
+      response {
+        version_info: "1"
+        nonce: "A"
+        type_url: "type.googleapis.com/envoy.config.listener.v3.Listener"
+        resources {
+          [type.googleapis.com/envoy.config.listener.v3.Listener] {
+            name: "server.example.com"
+            api_listener {
+              api_listener {
+                [type.googleapis.com/envoy.extensions.filters.network
+                     .http_connection_manager.v3.HttpConnectionManager] {
+                  http_filters {
+                    name: "router"
+                    typed_config {
+                      [type.googleapis.com/
+                       envoy.extensions.filters.http.router.v3.Router] {}
+                    }
+                  }
+                  rds {
+                    route_config_name: "route_config"
+                    config_source { self {} }
+                  }
+                }
+              }
+            }
+          }
+        }
+      }
+    }
+  }
+)pb";
 
-bool squelch = true;
+static const char* kBasicRouteConfig = R"pb(
+  bootstrap: "{\"xds_servers\": [{\"server_uri\":\"xds.example.com:443\", \"channel_creds\":[{\"type\": \"fake\"}]}]}"
+  actions {
+    start_watch {
+      resource_type { route_config {} }
+      resource_name: "route_config1"
+    }
+  }
+  actions {
+    read_message_from_client {
+      stream_id { ads {} }
+      ok: true
+    }
+  }
+  actions {
+    send_message_to_client {
+      stream_id { ads {} }
+      response {
+        version_info: "1"
+        nonce: "A"
+        type_url: "type.googleapis.com/envoy.config.route.v3.RouteConfiguration"
+        resources {
+          [type.googleapis.com/envoy.config.route.v3.RouteConfiguration] {
+            name: "route_config1"
+            virtual_hosts {
+              domains: "*"
+              routes {
+                match { prefix: "" }
+                route { cluster: "cluster1" }
+              }
+            }
+          }
+        }
+      }
+    }
+  }
+)pb";
 
-DEFINE_PROTO_FUZZER(const xds_client_fuzzer::Msg& message) {
-  grpc_core::Fuzzer fuzzer(message.bootstrap(),
-                           message.fuzzing_event_engine_actions());
+static const char* kBasicCluster = R"pb(
+  bootstrap: "{\"xds_servers\": [{\"server_uri\":\"xds.example.com:443\", \"channel_creds\":[{\"type\": \"fake\"}]}]}"
+  actions {
+    start_watch {
+      resource_type { cluster {} }
+      resource_name: "cluster1"
+    }
+  }
+  actions {
+    read_message_from_client {
+      stream_id { ads {} }
+      ok: true
+    }
+  }
+  actions {
+    send_message_to_client {
+      stream_id { ads {} }
+      response {
+        version_info: "1"
+        nonce: "A"
+        type_url: "type.googleapis.com/envoy.config.cluster.v3.Cluster"
+        resources {
+          [type.googleapis.com/envoy.config.cluster.v3.Cluster] {
+            name: "cluster1"
+            type: EDS
+            eds_cluster_config {
+              eds_config { ads {} }
+              service_name: "endpoint1"
+            }
+          }
+        }
+      }
+    }
+  }
+)pb";
+
+static const char* kBasicEndpoint = R"pb(
+  bootstrap: "{\"xds_servers\": [{\"server_uri\":\"xds.example.com:443\", \"channel_creds\":[{\"type\": \"fake\"}]}]}"
+  actions {
+    start_watch {
+      resource_type { endpoint {} }
+      resource_name: "endpoint1"
+    }
+  }
+  actions {
+    read_message_from_client {
+      stream_id { ads {} }
+      ok: true
+    }
+  }
+  actions {
+    send_message_to_client {
+      stream_id { ads {} }
+      response {
+        version_info: "1"
+        nonce: "A"
+        type_url: "type.googleapis.com/envoy.config.endpoint.v3.ClusterLoadAssignment"
+        resources {
+          [type.googleapis.com/envoy.config.endpoint.v3.ClusterLoadAssignment] {
+            cluster_name: "endpoint1"
+            endpoints {
+              locality { region: "region1" zone: "zone1" sub_zone: "sub_zone1" }
+              load_balancing_weight { value: 1 }
+              lb_endpoints {
+                load_balancing_weight { value: 1 }
+                endpoint {
+                  address {
+                    socket_address { address: "127.0.0.1" port_value: 443 }
+                  }
+                }
+              }
+            }
+          }
+        }
+      }
+    }
+  }
+)pb";
+
+auto ParseTestProto(const std::string& proto) {
+  xds_client_fuzzer::Msg msg;
+  CHECK(google::protobuf::TextFormat::ParseFromString(proto, &msg));
+  return msg;
+}
+
+void Fuzz(const xds_client_fuzzer::Msg& message) {
+  Fuzzer fuzzer(message.bootstrap(), message.fuzzing_event_engine_actions());
   for (int i = 0; i < message.actions_size(); i++) {
     fuzzer.Act(message.actions(i));
   }
 }
+FUZZ_TEST(XdsClientFuzzer, Fuzz)
+    .WithDomains(::fuzztest::Arbitrary<xds_client_fuzzer::Msg>().WithSeeds(
+        {ParseTestProto(kBasicCluster), ParseTestProto(kBasicEndpoint),
+         ParseTestProto(kBasicListener), ParseTestProto(kBasicRouteConfig)}));
+
+TEST(XdsClientFuzzer, XdsServersEmpty) {
+  Fuzz(ParseTestProto(R"pb(
+    bootstrap: "{\"xds_servers\": []}"
+    actions {
+      start_watch {
+        resource_type { listener {} }
+        resource_name: "\003"
+      }
+    }
+  )pb"));
+}
+
+TEST(XdsClientFuzzer, ResourceWrapperEmpty) {
+  Fuzz(ParseTestProto(R"pb(
+    bootstrap: "{\"xds_servers\": [{\"server_uri\":\"xds.example.com:443\", \"channel_creds\":[{\"type\": \"fake\"}]}]}"
+    actions { start_watch { resource_type { cluster {} } } }
+    actions {
+      send_message_to_client {
+        stream_id { ads {} }
+        response {
+          version_info: "envoy.config.cluster.v3.Cluster"
+          resources { type_url: "envoy.service.discovery.v3.Resource" }
+          canary: true
+          type_url: "envoy.config.cluster.v3.Cluster"
+          nonce: "envoy.config.cluster.v3.Cluster"
+        }
+      }
+    }
+  )pb"));
+}
+
+TEST(XdsClientFuzzer, RlsMissingTypedExtensionConfig) {
+  Fuzz(ParseTestProto(R"pb(
+    bootstrap: "{\"xds_servers\": [{\"server_uri\":\"xds.example.com:-257\", \"channel_creds\":[{\"type\": \"fake\"}]}]}"
+    actions { start_watch { resource_type { route_config {} } } }
+    actions {
+      send_message_to_client {
+        stream_id { ads {} }
+        response {
+          version_info: "grpc.lookup.v1.RouteLookup"
+          resources {
+            type_url: "envoy.config.route.v3.RouteConfiguration"
+            value: "\010\001b\000"
+          }
+          type_url: "envoy.config.route.v3.RouteConfiguration"
+          nonce: "/@\001\000\\\000\000x141183468234106731687303715884105729"
+        }
+      }
+    }
+  )pb"));
+}
+
+TEST(XdsClientFuzzer, SendMessageToClientBeforeStreamCreated) {
+  Fuzz(ParseTestProto(R"pb(
+    bootstrap: "{\"xds_servers\": [{\"server_uri\":\"xds.example.com:443\", \"channel_creds\":[{\"type\": \"fake\"}]}]}"
+    actions { send_message_to_client { stream_id { ads {} } } }
+  )pb"));
+}
+
+TEST(XdsClientFuzzer, IgnoresConnectionFailuresWithOkStatus) {
+  Fuzz(ParseTestProto(R"pb(
+    bootstrap: "{\"xds_servers\": [{\"server_uri\":\"xds.example.com\320\272443\", \"channel_creds\":[{\"type\": \"fake\"}]}]}"
+    actions {
+      start_watch {
+        resource_type { cluster {} }
+        resource_name: "*"
+      }
+    }
+    actions {}
+    actions { trigger_connection_failure {} }
+    actions {}
+    fuzzing_event_engine_actions { connections { write_size: 2147483647 } }
+  )pb"));
+}
+
+TEST(XdsClientFuzzer, UnsubscribeWhileAdsCallInBackoff) {
+  Fuzz(ParseTestProto(R"pb(
+    bootstrap: "{\"xds_servers\": [{\"server_uri\":\"xds.example.com:443\", \"channel_creds\":[{\"type\": \"fake\"}]}]}"
+    actions {
+      start_watch {
+        resource_type { listener {} }
+        resource_name: "server.example.com"
+      }
+    }
+    actions { send_status_to_client { stream_id { ads {} } } }
+    actions {
+      stop_watch {
+        resource_type { listener {} }
+        resource_name: "server.example.com"
+      }
+    }
+    actions {
+      send_message_to_client {
+        stream_id { ads {} }
+        response {
+          version_info: "1"
+          nonce: "A"
+          type_url: "type.googleapis.com/envoy.config.listener.v3.Listener"
+          resources {
+            [type.googleapis.com/envoy.config.listener.v3.Listener] {
+              name: "server.example.com"
+              api_listener {
+                api_listener {
+                  [type.googleapis.com/envoy.extensions.filters.network
+                       .http_connection_manager.v3.HttpConnectionManager] {
+                    http_filters {
+                      name: "router"
+                      typed_config {
+                        [type.googleapis.com/
+                         envoy.extensions.filters.http.router.v3.Router] {}
+                      }
+                    }
+                    rds {
+                      route_config_name: "route_config"
+                      config_source { self {} }
+                    }
+                  }
+                }
+              }
+            }
+          }
+        }
+      }
+    }
+  )pb"));
+}
+
+}  // namespace grpc_core
